@@ -43,11 +43,12 @@ int main
 	static const char *about = PACKAGE_STRING " " PACKAGE_URL;
 	static const char *version = PACKAGE_VERSION " (" PACKAGE_NAME ")";
 
-	std::vector<std::string>	peers;
-	std::string					passphrase, kfilename;
-	std::string					migrate_peer_from, migrate_peer_to;
-	int 						opt_verbose = 0;
-	char						*opt_k = NULL;
+	std::vector<std::string>		peers;
+	tmcg_openpgp_secure_string_t	passphrase;
+	std::string						kfilename;
+	std::string						migrate_peer_from, migrate_peer_to;
+	int 							opt_verbose = 0;
+	char							*opt_k = NULL;
 
 	// parse argument list
 	for (size_t i = 0; i < (size_t)(argc - 1); i++)
@@ -160,26 +161,17 @@ int main
 	}
 
 	// lock memory
+	bool force_secmem = false;
 	if (!lock_memory())
 	{
 		std::cerr << "WARNING: locking memory failed; CAP_IPC_LOCK required" <<
-			" for memory protection" << std::endl;
+			" for full memory protection" << std::endl;
 		// at least try to use libgcrypt's secure memory
-		if (!gcry_check_version(TMCG_LIBGCRYPT_VERSION))
-		{
-			std::cerr << "ERROR: libgcrypt version >= " <<
-				TMCG_LIBGCRYPT_VERSION << " required" << std::endl;
-			return -1;
-		}
-		gcry_control(GCRYCTL_SUSPEND_SECMEM_WARN);
-		gcry_control(GCRYCTL_USE_SECURE_RNDPOOL);
-		gcry_control(GCRYCTL_INIT_SECMEM, 16384, 0);
-		gcry_control(GCRYCTL_RESUME_SECMEM_WARN);
-		gcry_control(GCRYCTL_INITIALIZATION_FINISHED, 0);
+		force_secmem = true;
 	}
 
 	// initialize LibTMCG
-	if (!init_libTMCG())
+	if (!init_libTMCG(force_secmem))
 	{
 		std::cerr << "ERROR: initialization of LibTMCG failed" << std::endl;
 		return -1;
@@ -231,7 +223,7 @@ int main
 #ifdef DKGPG_TESTSUITE
 		passphrase = "Test";
 #else
-		if (!get_passphrase("Enter passphrase to unlock private key",
+		if (!get_passphrase("Enter passphrase to unlock private key", false,
 			passphrase))
 		{
 			std::cerr << "ERROR: cannot read passphrase" << std::endl;
@@ -255,7 +247,7 @@ int main
 		delete ring;
 		return -1;
 	}
-	if (!prv->pub->valid || prv->weak(opt_verbose))
+	if (!prv->pub->valid || prv->Weak(opt_verbose))
 	{
 		std::cerr << "ERROR: primary key is invalid or weak" << std::endl;
 		delete ring;
@@ -289,7 +281,7 @@ int main
 		(prv->private_subkeys[0]->pkalgo == TMCG_OPENPGP_PKALGO_EXPERIMENTAL9))
 	{
 		TMCG_OpenPGP_PrivateSubkey *sub = prv->private_subkeys[0];
-		if (!sub->pub->valid || sub->weak(opt_verbose))
+		if (!sub->pub->valid || sub->Weak(opt_verbose))
 		{
 			std::cerr << "ERROR: subkey is invalid or weak" << std::endl;
 			delete dss;
@@ -314,7 +306,7 @@ int main
 	CallasDonnerhackeFinneyShawThayerRFC4880::
 		KeyidCompute(prv->pub->pub_hashing, kid);
 	CallasDonnerhackeFinneyShawThayerRFC4880::
-		FingerprintCompute(prv->pub->pub_hashing, fpr);
+		FingerprintComputePretty(prv->pub->pub_hashing, fpr);
 	std::cout << "OpenPGP V4 Key ID of primary key: " << std::endl << "\t";
 	std::cout << kid << std::endl;
 	std::cout << "OpenPGP V4 fingerprint of primary key: " << std::endl << "\t";
@@ -339,7 +331,7 @@ int main
 		tmcg_openpgp_octets_t f(rk.key_fingerprint,
 			 rk.key_fingerprint+sizeof(rk.key_fingerprint));
 		CallasDonnerhackeFinneyShawThayerRFC4880::
-			FingerprintConvert(f, fpr);
+			FingerprintConvertPretty(f, fpr);
 		std::cout << "\t" << fpr << std::endl;
 	}
 	if (prv->pub->revkeys.size() == 0)
@@ -425,7 +417,7 @@ int main
 		CallasDonnerhackeFinneyShawThayerRFC4880::
 			KeyidCompute(sub->pub->sub_hashing, kid2);
 		CallasDonnerhackeFinneyShawThayerRFC4880::
-			FingerprintCompute(sub->pub->sub_hashing, fpr2);
+			FingerprintComputePretty(sub->pub->sub_hashing, fpr2);
 		std::cout << "OpenPGP V4 Key ID of subkey: " << std::endl << "\t";
 		std::cout << kid2 << std::endl;
 		std::cout << "OpenPGP V4 fingerprint of subkey: " << std::endl << "\t";
@@ -450,7 +442,7 @@ int main
 			tmcg_openpgp_octets_t f(rk.key_fingerprint,
 				 rk.key_fingerprint+sizeof(rk.key_fingerprint));
 			CallasDonnerhackeFinneyShawThayerRFC4880::
-				FingerprintConvert(f, fpr2);
+				FingerprintConvertPretty(f, fpr2);
 			std::cout << "\t" << fpr2 << std::endl;
 		}
 		if (sub->pub->revkeys.size() == 0)
