@@ -29,6 +29,7 @@
 // copy infos from DKGPG package before overwritten by GNUnet headers
 static const char *version = PACKAGE_VERSION " (" PACKAGE_NAME ")";
 static const char *about = PACKAGE_STRING " " PACKAGE_URL;
+static const char *protocol = "DKGPG-revoke-1.0";
 
 #include <sstream>
 #include <fstream>
@@ -57,14 +58,15 @@ pid_t 						pid[DKGPG_MAX_N];
 std::vector<std::string>	peers;
 bool						instance_forked = false;
 
-std::string					passphrase, kfilename;
-std::string					passwords, hostname, port;
+tmcg_openpgp_secure_string_t	passphrase;
+std::string						kfilename;
+std::string						passwords, hostname, port;
 
-int 						opt_verbose = 0;
-char						*opt_passwords = NULL;
-char						*opt_hostname = NULL;
-char						*opt_k = NULL;
-unsigned long int			opt_r = 0, opt_p = 55000, opt_W = 5;
+int 							opt_verbose = 0;
+char							*opt_passwords = NULL;
+char							*opt_hostname = NULL;
+char							*opt_k = NULL;
+unsigned long int				opt_r = 0, opt_p = 55000, opt_W = 5;
 
 void run_instance
 	(size_t whoami, const time_t sigtime, const size_t reason,
@@ -113,7 +115,7 @@ void run_instance
 #ifdef DKGPG_TESTSUITE
 		passphrase = "Test";
 #else
-		if (!get_passphrase("Enter passphrase to unlock private key",
+		if (!get_passphrase("Enter passphrase to unlock private key", false,
 			passphrase))
 		{
 			std::cerr << "ERROR: cannot read passphrase" << std::endl;
@@ -137,7 +139,7 @@ void run_instance
 		delete ring;
 		exit(-1);
 	}
-	if (!prv->pub->valid || prv->weak(opt_verbose))
+	if (!prv->pub->valid || prv->Weak(opt_verbose))
 	{
 		std::cerr << "ERROR: primary key is invalid or weak" << std::endl;
 		delete ring;
@@ -229,9 +231,11 @@ void run_instance
 		(opt_W * 60));
 			
 	// create an instance of a reliable broadcast protocol (RBC)
-	std::string myID = "dkg-revoke|";
+	std::string myID = "dkg-revoke|" + std::string(protocol) + "|";
 	for (size_t i = 0; i < peers.size(); i++)
 		myID += peers[i] + "|";
+	if (opt_verbose)
+		std::cerr << "RBC: myID = " << myID << std::endl;
 	// assume maximum asynchronous t-resilience for RBC
 	size_t T_RBC = (peers.size() - 1) / 3;
 	CachinKursawePetzoldShoupRBC *rbc =
@@ -328,15 +332,17 @@ void run_instance
 
 	// compute a hash of pub and sub, respectively
 	tmcg_openpgp_octets_t trailer_pub, pub_hashing, hash_pub, left_pub,
-		trailer_sub, sub_hashing, hash_sub, left_sub;
+		trailer_sub, sub_hashing, hash_sub, left_sub, issuer;
+	CallasDonnerhackeFinneyShawThayerRFC4880::
+		FingerprintCompute(prv->pub->pub_hashing, issuer);
 	CallasDonnerhackeFinneyShawThayerRFC4880::
 		PacketSigPrepareRevocationSignature(TMCG_OPENPGP_SIGNATURE_KEY_REVOCATION,
-			hashalgo, csigtime, revcode, "", prv->pub->id, trailer_pub);
+			hashalgo, csigtime, revcode, "", issuer, trailer_pub);
 	if (sub != NULL)
 	{
 		CallasDonnerhackeFinneyShawThayerRFC4880::
 			PacketSigPrepareRevocationSignature(TMCG_OPENPGP_SIGNATURE_SUBKEY_REVOCATION,
-				hashalgo, csigtime, revcode, "", prv->pub->id, trailer_sub);
+				hashalgo, csigtime, revcode, "", issuer, trailer_sub);
 	}
 	// RFC 4880 ERRATA:
 	// Primary key revocation signatures (type 0x20) hash only the key being
@@ -775,11 +781,11 @@ int main
 		),
 		GNUNET_GETOPT_OPTION_END
 	};
-	if (GNUNET_STRINGS_get_utf8_args(argc, argv, &argc, &argv) != GNUNET_OK)
-	{
-		std::cerr << "ERROR: GNUNET_STRINGS_get_utf8_args() failed" << std::endl;
-    		return -1;
-	}
+//	if (GNUNET_STRINGS_get_utf8_args(argc, argv, &argc, &argv) != GNUNET_OK)
+//	{
+//		std::cerr << "ERROR: GNUNET_STRINGS_get_utf8_args() failed" << std::endl;
+//		return -1;
+//	}
 	if (GNUNET_GETOPT_run(usage, options, argc, argv) == GNUNET_SYSERR)
 	{
 		std::cerr << "ERROR: GNUNET_GETOPT_run() failed" << std::endl;
@@ -921,24 +927,17 @@ int main
 	}
 
 	// lock memory
+	bool force_secmem = false;
 	if (!lock_memory())
 	{
-		std::cerr << "WARNING: locking memory failed; CAP_IPC_LOCK required for memory protection" << std::endl;
+		std::cerr << "WARNING: locking memory failed; CAP_IPC_LOCK required" <<
+			" for full memory protection" << std::endl;
 		// at least try to use libgcrypt's secure memory
-		if (!gcry_check_version(TMCG_LIBGCRYPT_VERSION))
-		{
-			std::cerr << "ERROR: libgcrypt version >= " << TMCG_LIBGCRYPT_VERSION << " required" << std::endl;
-			return -1;
-		}
-		gcry_control(GCRYCTL_SUSPEND_SECMEM_WARN);
-		gcry_control(GCRYCTL_USE_SECURE_RNDPOOL);
-		gcry_control(GCRYCTL_INIT_SECMEM, 16384, 0);
-		gcry_control(GCRYCTL_RESUME_SECMEM_WARN);
-		gcry_control(GCRYCTL_INITIALIZATION_FINISHED, 0);
+		force_secmem = true;
 	}
 
 	// initialize LibTMCG
-	if (!init_libTMCG())
+	if (!init_libTMCG(force_secmem))
 	{
 		std::cerr << "ERROR: initialization of LibTMCG failed" << std::endl;
 		return -1;
@@ -1032,7 +1031,7 @@ int main
 		GNUNET_GETOPT_OPTION_END
 	};
 	ret = GNUNET_PROGRAM_run(argc, argv, usage, about, myoptions, &gnunet_run, argv[0]);
-	GNUNET_free((void *) argv);
+//	GNUNET_free((void *) argv);
 	if (ret == GNUNET_OK)
 		return 0;
 	else
